@@ -9,8 +9,14 @@ import (
 
 // This is a compile-time assertion to ensure that this generated file
 // is compatible with the grpc package it is being compiled against.
-// Requires gRPC-Go v1.32.0 or later.
-const _ = grpc.SupportPackageIsVersion7
+// Requires gRPC-Go v1.64.0 or later.
+const _ = grpc.SupportPackageIsVersion9
+
+const (
+	InstanceManagementService_ListInstance_FullMethodName  = "/v2ray.core.app.instman.command.InstanceManagementService/ListInstance"
+	InstanceManagementService_AddInstance_FullMethodName   = "/v2ray.core.app.instman.command.InstanceManagementService/AddInstance"
+	InstanceManagementService_StartInstance_FullMethodName = "/v2ray.core.app.instman.command.InstanceManagementService/StartInstance"
+)
 
 // InstanceManagementServiceClient is the client API for InstanceManagementService service.
 //
@@ -30,8 +36,9 @@ func NewInstanceManagementServiceClient(cc grpc.ClientConnInterface) InstanceMan
 }
 
 func (c *instanceManagementServiceClient) ListInstance(ctx context.Context, in *ListInstanceReq, opts ...grpc.CallOption) (*ListInstanceResp, error) {
+	cOpts := append([]grpc.CallOption{grpc.StaticMethod()}, opts...)
 	out := new(ListInstanceResp)
-	err := c.cc.Invoke(ctx, "/v2ray.core.app.instman.command.InstanceManagementService/ListInstance", in, out, opts...)
+	err := c.cc.Invoke(ctx, InstanceManagementService_ListInstance_FullMethodName, in, out, cOpts...)
 	if err != nil {
 		return nil, err
 	}
@@ -39,8 +46,9 @@ func (c *instanceManagementServiceClient) ListInstance(ctx context.Context, in *
 }
 
 func (c *instanceManagementServiceClient) AddInstance(ctx context.Context, in *AddInstanceReq, opts ...grpc.CallOption) (*AddInstanceResp, error) {
+	cOpts := append([]grpc.CallOption{grpc.StaticMethod()}, opts...)
 	out := new(AddInstanceResp)
-	err := c.cc.Invoke(ctx, "/v2ray.core.app.instman.command.InstanceManagementService/AddInstance", in, out, opts...)
+	err := c.cc.Invoke(ctx, InstanceManagementService_AddInstance_FullMethodName, in, out, cOpts...)
 	if err != nil {
 		return nil, err
 	}
@@ -48,8 +56,9 @@ func (c *instanceManagementServiceClient) AddInstance(ctx context.Context, in *A
 }
 
 func (c *instanceManagementServiceClient) StartInstance(ctx context.Context, in *StartInstanceReq, opts ...grpc.CallOption) (*StartInstanceResp, error) {
+	cOpts := append([]grpc.CallOption{grpc.StaticMethod()}, opts...)
 	out := new(StartInstanceResp)
-	err := c.cc.Invoke(ctx, "/v2ray.core.app.instman.command.InstanceManagementService/StartInstance", in, out, opts...)
+	err := c.cc.Invoke(ctx, InstanceManagementService_StartInstance_FullMethodName, in, out, cOpts...)
 	if err != nil {
 		return nil, err
 	}
@@ -58,7 +67,7 @@ func (c *instanceManagementServiceClient) StartInstance(ctx context.Context, in
 
 // InstanceManagementServiceServer is the server API for InstanceManagementService service.
 // All implementations must embed UnimplementedInstanceManagementServiceServer
-// for forward compatibility
+// for forward compatibility.
 type InstanceManagementServiceServer interface {
 	ListInstance(context.Context, *ListInstanceReq) (*ListInstanceResp, error)
 	AddInstance(context.Context, *AddInstanceReq) (*AddInstanceResp, error)
@@ -66,9 +75,12 @@ type InstanceManagementServiceServer interface {
 	mustEmbedUnimplementedInstanceManagementServiceServer()
 }
 
-// UnimplementedInstanceManagementServiceServer must be embedded to have forward compatible implementations.
-type UnimplementedInstanceManagementServiceServer struct {
-}
+// UnimplementedInstanceManagementServiceServer must be embedded to have
+// forward compatible implementations.
+//
+// NOTE: this should be embedded by value instead of pointer to avoid a nil
+// pointer dereference when methods are called.
+type UnimplementedInstanceManagementServiceServer struct{}
 
 func (UnimplementedInstanceManagementServiceServer) ListInstance(context.Context, *ListInstanceReq) (*ListInstanceResp, error) {
 	return nil, status.Errorf(codes.Unimplemented, "method ListInstance not implemented")
@@ -81,6 +93,7 @@ func (UnimplementedInstanceManagementServiceServer) StartInstance(context.Contex
 }
 func (UnimplementedInstanceManagementServiceServer) mustEmbedUnimplementedInstanceManagementServiceServer() {
 }
+func (UnimplementedInstanceManagementServiceServer) testEmbeddedByValue() {}
 
 // UnsafeInstanceManagementServiceServer may be embedded to opt out of forward compatibility for this service.
 // Use of this interface is not recommended, as added methods to InstanceManagementServiceServer will
@@ -90,6 +103,13 @@ type UnsafeInstanceManagementServiceServer interface {
 }
 
 func RegisterInstanceManagementServiceServer(s grpc.ServiceRegistrar, srv InstanceManagementServiceServer) {
+	// If the following call pancis, it indicates UnimplementedInstanceManagementServiceServer was
+	// embedded by pointer and is nil.  This will cause panics if an
+	// unimplemented method is ever invoked, so we test this at initialization
+	// time to prevent it from happening at runtime later due to I/O.
+	if t, ok := srv.(interface{ testEmbeddedByValue() }); ok {
+		t.testEmbeddedByValue()
+	}
 	s.RegisterService(&InstanceManagementService_ServiceDesc, srv)
 }
 
@@ -103,7 +123,7 @@ func _InstanceManagementService_ListInstance_Handler(srv interface{}, ctx contex
 	}
 	info := &grpc.UnaryServerInfo{
 		Server:     srv,
-		FullMethod: "/v2ray.core.app.instman.command.InstanceManagementService/ListInstance",
+		FullMethod: InstanceManagementService_ListInstance_FullMethodName,
 	}
 	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
 		return srv.(InstanceManagementServiceServer).ListInstance(ctx, req.(*ListInstanceReq))
@@ -121,7 +141,7 @@ func _InstanceManagementService_AddInstance_Handler(srv interface{}, ctx context
 	}
 	info := &grpc.UnaryServerInfo{
 		Server:     srv,
-		FullMethod: "/v2ray.core.app.instman.command.InstanceManagementService/AddInstance",
+		FullMethod: InstanceManagementService_AddInstance_FullMethodName,
 	}
 	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
 		return srv.(InstanceManagementServiceServer).AddInstance(ctx, req.(*AddInstanceReq))
@@ -139,7 +159,7 @@ func _InstanceManagementService_StartInstance_Handler(srv interface{}, ctx conte
 	}
 	info := &grpc.UnaryServerInfo{
 		Server:     srv,
-		FullMethod: "/v2ray.core.app.instman.command.InstanceManagementService/StartInstance",
+		FullMethod: InstanceManagementService_StartInstance_FullMethodName,
 	}
 	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
 		return srv.(InstanceManagementServiceServer).StartInstance(ctx, req.(*StartInstanceReq))
