@@ -1,13 +1,8 @@
-//go:build !confonly
-// +build !confonly
-
 package log
 
 import (
-	"sync"
-
-	"github.com/v2fly/v2ray-core/v4/common"
-	"github.com/v2fly/v2ray-core/v4/common/log"
+	"github.com/v2fly/v2ray-core/v5/common"
+	"github.com/v2fly/v2ray-core/v5/common/log"
 )
 
 type HandlerCreatorOptions struct {
@@ -18,24 +13,16 @@ type HandlerCreator func(LogType, HandlerCreatorOptions) (log.Handler, error)
 
 var handlerCreatorMap = make(map[LogType]HandlerCreator)
 
-var handlerCreatorMapLock = &sync.RWMutex{}
-
 func RegisterHandlerCreator(logType LogType, f HandlerCreator) error {
 	if f == nil {
 		return newError("nil HandlerCreator")
 	}
 
-	handlerCreatorMapLock.Lock()
-	defer handlerCreatorMapLock.Unlock()
-
 	handlerCreatorMap[logType] = f
 	return nil
 }
 
 func createHandler(logType LogType, options HandlerCreatorOptions) (log.Handler, error) {
-	handlerCreatorMapLock.RLock()
-	defer handlerCreatorMapLock.RUnlock()
-
 	creator, found := handlerCreatorMap[logType]
 	if !found {
 		return nil, newError("unable to create log handler for ", logType)
