@@ -2,7 +2,7 @@ package environment
 
 import (
 	"github.com/v2fly/v2ray-core/v5/common/environment/filesystemcap"
-	"github.com/v2fly/v2ray-core/v5/common/log"
+	"github.com/v2fly/v2ray-core/v5/features/extension/storage"
 	"github.com/v2fly/v2ray-core/v5/transport/internet"
 	"github.com/v2fly/v2ray-core/v5/transport/internet/tagged"
 )
@@ -27,13 +27,24 @@ type InstanceNetworkCapabilitySet interface {
 }
 
 type FeaturesLookupCapabilitySet interface {
-	RequireFeatures(callback interface{}) error
+	RequireFeatures() interface{}
 }
 
 type LogCapabilitySet interface {
-	RecordLog(msg log.Message)
+	RecordLog() interface{}
 }
 
 type FileSystemCapabilitySet interface {
 	filesystemcap.FileSystemCapabilitySet
 }
+
+type PersistentStorageCapabilitySet interface {
+	PersistentStorage() storage.ScopedPersistentStorage
+}
+type TransientStorageCapabilitySet interface {
+	TransientStorage() storage.ScopedTransientStorage
+}
+
+type ProxyMetadataCapabilitySet interface {
+	SelfProxyTag() string
+}
