@@ -5,6 +5,7 @@ import (
 	gonet "net"
 
 	"github.com/v2fly/v2ray-core/v5/common/buf"
+	"github.com/v2fly/v2ray-core/v5/common/errors"
 	"github.com/v2fly/v2ray-core/v5/common/net"
 	"github.com/v2fly/v2ray-core/v5/common/protocol"
 )
@@ -45,6 +46,9 @@ func ExtractAddressFromPacket(data *buf.Buffer) (*buf.Buffer, gonet.Addr, error)
 	if err != nil {
 		return nil, nil, err
 	}
+	if address.Family().IsDomain() {
+		return nil, nil, errors.New("invalid address type")
+	}
 	addr := &gonet.UDPAddr{
 		IP:   address.IP(),
 		Port: int(port.Value()),
