@@ -57,6 +57,8 @@ type Handler struct {
 	timeout         time.Duration
 
 	config *Config
+
+	nonIPQuery string
 }
 
 func (h *Handler) Init(config *Config, dnsClient dns.Client, policyManager policy.Manager) error {
@@ -88,6 +90,8 @@ func (h *Handler) Init(config *Config, dnsClient dns.Client, policyManager polic
 	}
 
 	h.config = config
+
+	h.nonIPQuery = config.Non_IPQuery
 	return nil
 }
 
@@ -200,15 +204,16 @@ func (h *Handler) Process(ctx context.Context, link *transport.Link, d internet.
 
 			if !h.isOwnLink(ctx) {
 				isIPQuery, domain, id, qType := parseIPQuery(b.Bytes())
-				if isIPQuery {
+				if isIPQuery || h.nonIPQuery != "drop" {
 					if domain, err := strmatcher.ToDomain(domain); err == nil {
 						go h.handleIPQuery(id, qType, domain, writer)
-						continue
+					} else {
+						h.handleDNSError(id, dnsmessage.RCodeFormatError, writer)
 					}
+				} else {
+					h.handleDNSError(id, dnsmessage.RCodeNotImplemented, writer)
 				}
-			}
-
-			if err := connWriter.WriteMessage(b); err != nil {
+			} else if err := connWriter.WriteMessage(b); err != nil {
 				return err
 			}
 		}
@@ -305,6 +310,35 @@ func (h *Handler) handleIPQuery(id uint16, qType dnsmessage.Type, domain string,
 	}
 }
 
+func (h *Handler) handleDNSError(id uint16, rCode dnsmessage.RCode, writer dns_proto.MessageWriter) {
+	var err error
+
+	b := buf.New()
+	rawBytes := b.Extend(buf.Size)
+	builder := dnsmessage.NewBuilder(rawBytes[:0], dnsmessage.Header{
+		ID:                 id,
+		RCode:              rCode,
+		RecursionAvailable: true,
+		RecursionDesired:   true,
+		Response:           true,
+	})
+	builder.EnableCompression()
+	common.Must(builder.StartQuestions())
+	common.Must(builder.StartAnswers())
+
+	msgBytes, err := builder.Finish()
+	if err != nil {
+		newError("pack message").Base(err).WriteToLog()
+		b.Release()
+		return
+	}
+	b.Resize(0, int32(len(msgBytes)))
+
+	if err := writer.WriteMessage(b); err != nil {
+		newError("write IP answer").Base(err).WriteToLog()
+	}
+}
+
 type outboundConn struct {
 	access sync.Mutex
 	dialer func() (internet.Connection, error)
