@@ -1,13 +1,16 @@
 package internet
 
 import (
+	"net"
 	"syscall"
 
 	"golang.org/x/sys/windows"
 )
 
 const (
-	TCP_FASTOPEN = 15 // nolint: revive,stylecheck
+	TCP_FASTOPEN    = 15 // nolint: revive,stylecheck
+	IP_UNICAST_IF   = 31 // nolint: revive,stylecheck
+	IPV6_UNICAST_IF = 31 // nolint: revive,stylecheck
 )
 
 func setTFO(fd syscall.Handle, settings SocketConfig_TCPFastOpenState) error {
@@ -36,6 +39,19 @@ func applyOutboundSocketOptions(network string, address string, fd uintptr, conf
 		}
 	}
 
+	if config.BindToDevice != "" {
+		iface, err := net.InterfaceByName(config.BindToDevice)
+		if err != nil {
+			return newError("failed to get interface ", config.BindToDevice).Base(err)
+		}
+		if err := windows.SetsockoptInt(windows.Handle(fd), windows.IPPROTO_IP, IP_UNICAST_IF, iface.Index); err != nil {
+			return newError("failed to set IP_UNICAST_IF", err)
+		}
+		if err := windows.SetsockoptInt(windows.Handle(fd), windows.IPPROTO_IPV6, IPV6_UNICAST_IF, iface.Index); err != nil {
+			return newError("failed to set IPV6_UNICAST_IF", err)
+		}
+	}
+
 	if config.TxBufSize != 0 {
 		if err := windows.SetsockoptInt(windows.Handle(fd), windows.SOL_SOCKET, windows.SO_SNDBUF, int(config.TxBufSize)); err != nil {
 			return newError("failed to set SO_SNDBUF").Base(err)
